
/************************************************************************/
/*                                                                      */
/* Juan Quiroz and Dat Ta                                               */
/* CS 660 - Compiler Construction                                       */
/*                                                                      */
/* Main driver:
   Initializes global variables, parses the command line, initializes
   file pointers, calls yyparse, and closes file pointers.
*/
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "derivednodes.h"
#include "archivereg.h"

#define DEBUGLEVELS 5

enum {DBGLV0, DBGLV1, DBGLV2, DBGLV3, DBGLV4};

#define INSERTMODE 0
#define LOOKUPMODE 1

#define OVERFLOW 60

/********************
 * On the current code the source code is specified with
 * the -c source_code option.
**********************/

/*********************************
  Function Declarations
*********************************/
void Usage();
/*********************************/

/*********************************
 Global variables
*********************************/
int scannerDebugLevel=1;
int parserDebugLevel=1;
int stDebugLevel=2;
int warningsDebugLevel=1;

FILE *fp=NULL;

// AST file handle
FILE *f_asm=NULL;

int INTOVERFLOW = OVERFLOW;
int FLOATOVERFLOW = OVERFLOW;
int MAXVARNAMESIZE = OVERFLOW;

/*********************************/


/*********************************
// Driver
*********************************/
int main(int argc, char *argv[])
{
    int c=0;
    char *sourceCode=NULL;

    // AST dump file name
    char *asmDump=NULL;

    ASTNode *astroot= new PROGNode();

    // Check if there's any input to the compiler
    if ( argc <= 1 )
    {
        printf("No input files!\n");
        return 0;
    }

    // Parse the command line
    //while ( (c = getopt(argc, argv, "d:o:hc:s:pty")) != EOF )
    while ( (c = getopt(argc, argv, "o:hc:s:p:t:y:wdf:i:j:a:")) != EOF )
    {
        switch (c) 
        {
            case 'w':
                warningsDebugLevel = DBGLV0;
                break;

            case 'j':
                MAXVARNAMESIZE = atoi(optarg);
                if ( MAXVARNAMESIZE < 1 )
                    MAXVARNAMESIZE = 2;
                printf("Defaulting variable name size to %d.\n", MAXVARNAMESIZE);
                break;

            case 'i':
                INTOVERFLOW = atoi(optarg);
                if ( INTOVERFLOW < 1 )
                    INTOVERFLOW = 2;
                printf("Defaulting integer overflow to %d.\n", INTOVERFLOW);
                break;

            case 'f':
                FLOATOVERFLOW = atoi(optarg);
                if ( FLOATOVERFLOW < 1 )
                    FLOATOVERFLOW = 2;
                printf("Defaulting float overflow to %d.\n", FLOATOVERFLOW);
                break;

            case 't':
                //scanner debug level
                // 0 no output
                // 1 all output
                // 2 reserved words
                // 3 identifiers
                // 4 output to own file
                scannerDebugLevel = atoi(optarg);
                if ( scannerDebugLevel < 0 )
                    scannerDebugLevel = DBGLV0;
                break;

            case 'p':
                //parser debug level
                // 0 no output
                // 1 all output
                // 2 output to own file
                parserDebugLevel = atoi(optarg);
                if ( parserDebugLevel < 0 )
                    parserDebugLevel = DBGLV0;
                break;

            case 'y':
                //parser debug level
                // 0 no output
                // 1 all output
                // 2 all output with var values
                stDebugLevel = atoi(optarg);
                if ( stDebugLevel < 0 )
                    stDebugLevel = DBGLV0;
                break;

            case 'a':
                // ASM Dump Output file name
                asmDump = new char[strlen(optarg)+1];
                strcpy(asmDump, optarg);
                break;

            case 'c':
                // source code file name
                sourceCode = new char[strlen(optarg)+1];
                strcpy(sourceCode, optarg);
                break;
                
            case 'h':
                // help flag 
                Usage();
                exit(1);
                break;

            default:
                // If erroneous input then display usage of compiler
                Usage();
                exit(1);
        }
    }

    // if source code not provided
    if ( sourceCode == NULL )
    {
        printf("No input source code file!!\n");
        Usage();
        exit(1);
    }

    // if ASM output file not provided
    if ( asmDump == NULL )
    {
        printf("Writing ASM output to file \"asmcode.asm\".\n");
        asmDump = new char[8];
        strcpy(asmDump,"asmcode.asm");
    }


    printf("\n");

    /////////////////////////////////////////////////////
    // Open file handles
    /////////////////////////////////////////////////////
    // file pointers for source code
    fp = fopen(sourceCode, "r");
    // if source code file is not valid
    if ( fp == NULL )
    {
        printf("Error: source code file does not exist.\n");
        exit(1);
    }
    else
        fclose(fp);

    f_asm = fopen(asmDump, "w");
    /////////////////////////////////////////////////////

    {
        ifstream ifs(sourceCode);
        boost::archive::text_iarchive ia(ifs);
        register_all(ia);
        ia >> BOOST_SERIALIZATION_NVP(astroot);
    }

    astroot->DumpOut(f_asm);

    // close file pointers
    fclose(f_asm);

    delete astroot;

    return 0;
}

void Usage()
{
    printf("\nRunning the compiler:\n\
            compiler [OPTIONS] -c sourceCode\n\
            OPTIONS\n\
            -o scanner_parser_output_file\n\
                \tSpecifies the output file for the parser and scanner dumps.\n\
            -s symbol_table_output_file\n\
                \tSpecifies the output file for the symbol table dumps.\n\
            -p [0-2]\t\tParser debug level.\n\
            -t [0-4]\t\tScanner debug level.\n\
            -y [0-2]\t\tSymbol table debug level.\n\
            -w \t\tDisplay all warnings.\n\
            -h \t\tDisplay help menu.\n\
            ex. compiler -o outputFile -c sourceCode\n\n");
}
