
#include "astnode.h"

int tabCount=0;

// Constructor with default parameters
// If no parameters are passed, then the parameters are set to ""
ASTNode::ASTNode(string _type, string _name, string _value, string _type_specifier)
{
    name=_name;
    type=_type;
    value=_value;
    type_specifier=_type_specifier;

    actualValue._int=0;
    actualValue._float=0.0;

    paramSize=-1;
}

ASTNode::~ASTNode()
{
    clear();
}

void ASTNode::clear()
{
    // Delete all children
    for ( int i=0; i < children.size(); ++i )
    {
        if (children[i] != NULL )
            delete children[i];
    }

    // clear children vector
    children.clear();
}

string ASTNode::getType()
{
    return type;
}

string ASTNode::getTypeSpecifier()
{
    return type_specifier;
}

string ASTNode::getName()
{
    return name;
}

string ASTNode::getValue()
{
    return value;
}

string ASTNode::getStartLabel()
{
    return start_label;
}

string ASTNode::getEndLabel()
{
    return end_label;
}

void ASTNode::setStartLabel(string _label)
{
    start_label  = _label;
}

void ASTNode::setEndLabel(string _label)
{
    end_label = _label;
}

void ASTNode::setStartLabel(char _label[10])
{
    start_label  = _label;
}

void ASTNode::setEndLabel(char _label[10])
{
    end_label = _label;
}


bool ASTNode::addLeaf(ASTNode *leaf)
{
    if ( leaf != NULL )
    {
        children.push_back(leaf);
        return true;
    }
    else
    {
        printf("Warning: Adding NULL leaf to AST!!\n");
        return false;
    }
}

void ASTNode::addBranch(vector<ASTNode *>*branch)
{
    for ( int i=0; i < branch->size(); ++i )
    {
        if ( branch->at(i) != NULL )
            children.push_back(branch->at(i));
        else
            printf("Warning: Adding NULL leaf in branch to AST!!\n");
    }
}

int ASTNode::getChildrenNum()
{
    return children.size();
}

// Sets indenting tab
string ASTNode::setIndentTab()
{
    string tab="";
    for ( int i=0; i < tabCount; ++i )
        tab = tab+"    ";
        //tab = tab+"\t";

    return tab;
}

void ASTNode::DumpOut(FILE * fout)
{

    string tab=setIndentTab();

    fprintf(fout, "%s<%s>\n", tab.c_str(), type.c_str());

    ++tabCount;
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;

    fprintf(fout, "%s</%s>\n", tab.c_str(), type.c_str());
}

void ASTNode::ReadIn(FILE *)
{
}

ASTNode* ASTNode::getChild(int index)
{
    if ( index < children.size() )
        return children[index];
    else
    {
        fprintf(stderr, "Parameter index out of range!\n");
        return NULL;
    }
}

void ASTNode::setValue(string _value)
{
    value=_value;
}

void ASTNode::setValue(char *_value)
{
    value=_value;
}

void ASTNode::setName(string _name)
{
    name=_name;
}

void ASTNode::setName(char *_name)
{
    name=_name;
}

void ASTNode::setTypeSpecifier(char *_type_specifier)
{
    type_specifier=_type_specifier;
}

void ASTNode::setTypeSpecifier(string _type_specifier)
{
    type_specifier=_type_specifier;
}

void ASTNode::setType(string _type)
{
    type=_type;
}

void ASTNode::setType(char *_type)
{
    type=_type;
}

vector<ASTNode *>* ASTNode::getChildrenPtr()
{
    return &(children);
}
