#ifndef ASTNODE_H
#define ASTNODE_H

#include "rbtree.h"

#include <fstream>

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>

#include <boost/serialization/serialization.hpp>
#include <boost/serialization/utility.hpp>
#include <boost/serialization/base_object.hpp>
#include <boost/serialization/vector.hpp>

struct FUNCParam
{
    string name;
    string type;
    string value;
    int size;

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & name;
            ar & type;
            ar & value;
            ar & size;
        }
        // End code for serialization
};

struct Declaration
{
    string name;
    int type;
    string value;

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & name;
            ar & type;
            ar & value;
        }
        // End code for serialization
};

struct Value
{
    int _int;
    float _float;
    char _char;

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & _int;
            ar & _float;
            ar & _char;
        }
        // End code for serialization
};

class ASTNode
{
    public:

        ASTNode(string _type="", string _name="", string _value="", string _type_specifier="");
        virtual ~ASTNode();

        virtual void Code_Gen(FILE *fout);
        virtual void DumpOut(FILE *);
        virtual void ReadIn(FILE *);

        string getStartLabel();
        string getEndLabel();
        string getValue();
        string getName();
        string getType();
        string getTypeSpecifier();

        // Adds a child to the node
        bool addLeaf(ASTNode *leaf);
        void addBranch(vector<ASTNode *>*branch);

        // Gets the number of children
        int getChildrenNum();
        ASTNode *getChild(int index);

        void setStartLabel(string _label);
        void setEndLabel(string _label);

        void setStartLabel(char _label[10]);
        void setEndLabel(char _label[10]);
        void setValue(char *_value);
        void setValue(string _value);
        void setType(char *_type);
        void setType(string _type);
        void setTypeSpecifier(char *_type);
        void setTypeSpecifier(string _type);
        void setName(char *_name);
        void setName(string _name);

        // Sets up indenting tab
        string setIndentTab();

        void clear();

        vector<ASTNode *>* getChildrenPtr();

        int paramSize;

    protected:
        string name;
        // Holds a string representation of the type
        // i.e. "ADD", "ARITHMETIC", "FUNC", "INT"
        // i.e. "ADD", "ARITHMETIC", "FUNC", "VAR"
        string type;
        // Holds any of the predefined types in the grammar, if any, of the node
        string type_specifier;

	//start label is for code_gen when it needs to jump to the beginning
	string start_label;
	//end label is for code_gen when it needs to jump to the end (of loop) 
	string end_label;

        // Holds the string representation of the value
        string value;
        // Holds the actual value, if any, of the node
        Value actualValue;

        // Children nodes
        vector<ASTNode *> children;

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            ar & name;
            ar & type;
            ar & type_specifier;
            ar & value;
            ar & actualValue;
            ar & children;
            ar & paramSize;
        }
};

#endif
