
#include "derivednodes.h"
#include "regalloc.h"
extern RegisterAllocator *regAlloc;
extern Stack *codegen_st;
extern int labelNum;
static node *tempNode=NULL;
bool func=false;
string funcName;

//-----------------------------------------------------------------------//
//////////////////////////////////////////////////////////////
// Base AST Node Implementation                             //
//////////////////////////////////////////////////////////////

void ASTNode::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}


//-----------------------------------------------------------------------//
//////////////////////////////////////////////////////////////
// HIGH LEVEL NODES                                         //
//////////////////////////////////////////////////////////////

void DECL_Node::Code_Gen(FILE *fout)
{
    fprintf(fout,"\t\n");
    fprintf(fout,"#########################\n");
    fprintf(fout,"#Start Decls\n");
    IDENTIFIER_Node* child_i=NULL;
    int tempSize=0;


    // Declare global variables
    fprintf(fout, "\t#### Global variables\n");
    if ( dynamic_cast<IDENTIFIER_Node*>(children[0])->getScope() == "global")
    {
        node *node_temp=NULL;
        for ( int i=0; i < children.size(); ++i )
        {
            child_i = dynamic_cast<IDENTIFIER_Node*>(children[i]);
            tempSize = child_i->getSize();
            fprintf(fout, "\t.comm\t%s %d\n", child_i->getName().c_str(),
                    tempSize == -1 ? 4: tempSize*4);

            // Store variables in symbol table
            node_temp=new node(0,children[i]->getName(),_INT, tempSize);
            node_temp->setScope(GLOBAL);
            codegen_st->insert(node_temp);
        }
    }

    fprintf(fout,"#########################\n");

}

//////////////////////////////////////////////////////////////
// Function Code Generation
//////////////////////////////////////////////////////////////
void FUNC_Node::Code_Gen(FILE *fout)
{
    /*
    node *node_temp=new node(0,name,_FUNCTION, -1);
    node_temp->setScope(FUNC);
    codegen_st->insert(node_temp);
    */

    fprintf(fout, "\t.globl %s\n", name.c_str());
    fprintf(fout, "\t.ent %s\n", name.c_str());
    fprintf(fout, "%s:\n", name.c_str());
    prologue(fout);


    fprintf(fout, "### Function body begin\n");
    for ( int i=0; i < children.size(); ++i )
    {
        children[i]->Code_Gen(fout);
    }
    fprintf(fout, "### Function body end\n");

    epilogue(fout);

}

void FUNC_Node::prologue(FILE *fout)
{
    codegen_st->push();

    int tempSize=0;
    int declLoc=0;
    bool found=false;
    node *node_temp=NULL;

    // Find declaration list
    for ( int i=0; i < decls.size(); ++i )
    {
        if ( decls[i]->getType() == "Declaration_list" )
        {
            declLoc=i;
            found=true;
            i=decls.size();
        }
    }

    // For every decl, compute its size and add it to total
    // The first offset is for the return register
    stackSize=4;

    // Assumming we're only passing two parameters at the most
    if (params.size() == 1 )
    {
        // compute stack size
        stackSize += params[0].size == -1 ? 4: params[0].size*4;
        node_temp=new node(0,params[0].name,_INT,params[0].size);
        node_temp->setScope(PARAM1);
        codegen_st->insert(node_temp);
    }
    else if ( params.size() == 2 )
    {
        // compute stack size
        stackSize += params[0].size == -1 ? 4: params[0].size*4;
        node_temp=new node(0,params[0].name,_INT,params[0].size);
        node_temp->setScope(PARAM1);
        codegen_st->insert(node_temp);

        stackSize += params[1].size == -1 ? 4: params[1].size*4;
        node_temp=new node(0,params[1].name,_INT,params[1].size);
        node_temp->setScope(PARAM2);
        codegen_st->insert(node_temp);
    }
    stackSize = 16;

    if ( found )
    {
        // Get list of decls
        vector<ASTNode*>* declPtr = decls[declLoc]->getChildrenPtr();
        vector<ASTNode*>* idenPtr = NULL;
        int j=0;
        vector <node*>declNodes;

        for ( int i=0; i < declPtr->size(); ++i )
        {
            // I know type_specifier is int, since we're only limited
            // to integers, I should have used the typeMap to map
            // between ints and strings, but it's too late now
            // I won't call this for now, but I should if I'm to expand
            //stackSize+=declPtr->at(i)->getTypeSpecifier();

            // iterate throught identifier nodes in function declaration
            // section
            idenPtr = declPtr->at(i)->getChildrenPtr();
            // identifier children list
            for ( j=0; j < idenPtr->size(); ++j )
            {
                tempSize = dynamic_cast<IDENTIFIER_Node*>(idenPtr->at(j))->getSize();

                // Store variables in symbol table
                node_temp=new node(0,idenPtr->at(j)->getName(),_INT,tempSize );
                node_temp->setScope(LOCAL);

                // compute stack size
                stackSize += tempSize == -1 ? 4: tempSize*4;
                node_temp->setOffset(stackSize);
                declNodes.push_back(node_temp);
            }
        }

        // Add 4 just in case
        stackSize+=4;
        // Make stack size a multiple of 8
        if ( stackSize%8 )
            stackSize += stackSize%8;

        if ( stackSize < 32 )
            stackSize = 32;

        // Correct stack offsets
        for ( int i=0; i < declNodes.size(); ++i )
        {
            declNodes[i]->setOffset(stackSize-declNodes[i]->getOffset());
            codegen_st->insert(declNodes[i]);
        }
        // Clear pointers, leave memory in ST
        declNodes.clear();
    }
    else
        stackSize = 32;


    fprintf(fout,"\n\t# Function Prologue\n");
    // Create stackframe
    fprintf(fout,"\tsubu\t$sp, %d\n", stackSize);
    // Save the return address
    fprintf(fout,"\tsw\t$31, %d($sp)\n", stackSize-4);

    if ( name != "main" )
    {
        // Save function args
        fprintf(fout,"\t# Save function args\n", stackSize-4);
        fprintf(fout,"\tsw\t$4, %d($sp)\n", stackSize-8);
        fprintf(fout,"\tsw\t$5, %d($sp)\n", stackSize-12);
//        fprintf(fout,"\tsw\t$2, %d($sp)\n", stackSize-16);
    }

    // Only $31 was saved at ($sp)+stackSize-4
    fprintf(fout,"\t.mask\t0x80000000, -4\n");
    // define frame size and return register
    fprintf(fout,"\t.frame\t$sp, %d, $31\n", stackSize);

}

void FUNC_Node::epilogue(FILE *fout)
{
    if ( name != "main" )
    {
        // Retrieve function args
        fprintf(fout,"\t# Retrieve function args\n", stackSize-4);
        fprintf(fout,"\tlw\t$4, %d($sp)\n", stackSize-8);
        fprintf(fout,"\tlw\t$5, %d($sp)\n", stackSize-12);
 //       fprintf(fout,"\tlw\t$2, %d($sp)\n", stackSize-16);
    }

    fprintf(fout,"\n\t# Function Epilogue\n");
    fprintf(fout,"\tlw\t$31, %d($sp)    ## Restore return address\n", stackSize-4);
    fprintf(fout,"\taddu\t$sp, %d       ## Delete stackframe\n", stackSize);

    if ( name == "main")
    {
        fprintf(fout, "\tli\t$v0, 10\n");
        fprintf(fout, "\tsyscall\n");
    }
    else
    {
        fprintf(fout, "\tj\t$31\n");
    }
    fprintf(fout, "\t.end %s\n\n", name.c_str());


    //if ( !codegen_st->empty() )
        // Pop scope for function local variables

    rbtree *popTree = codegen_st->pop();
    popTree->clear();
}

//////////////////////////////////////////////////////////////
// Program Code Generation
//////////////////////////////////////////////////////////////
void PROGNode::Code_Gen(FILE *fout)
{
    codegen_st->push();

    // Code generation for  decls
    for ( int i=0; i < decls.size(); ++i )
        decls[i]->Code_Gen(fout);

    node *tempNode;
    // Insert function names into ST
    for ( int i=0; i < funcs.size(); ++i )
    {
        tempNode = new node(0, funcs[i]->getName(), -1);
        tempNode->setScope(LOCAL);
        codegen_st->insert(tempNode);
    }

    // Code generation for functions in program
    for ( int i=0; i < funcs.size(); ++i )
    {
        funcs[i]->Code_Gen(fout);
    }

}

//-----------------------------------------------------------------------//

//////////////////////////////////////////////////////////////
// DERIVED LEVEL NODES                                      //
//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////
// Constant Code Generation
//////////////////////////////////////////////////////////////
void CONSTANT_Node::Code_Gen(FILE *fout)
{
    regAlloc->Get_Register(value, "constant");
}

//////////////////////////////////////////////////////////////
// Identifier Code Generation
//////////////////////////////////////////////////////////////
void IDENTIFIER_Node::Code_Gen(FILE *fout)
{
    // If function call, just get name of function
    if ( func )
    {
        funcName=name;
     //   regAlloc->Get_Register(name, "func");
    }
    else
        regAlloc->Get_Register(name, scope);
}

//////////////////////////////////////////////////////////////
// String Code Generation
//////////////////////////////////////////////////////////////
void STRING_Node::Code_Gen(FILE *fout)
{
}

//////////////////////////////////////////////////////////////
// Postfix expression Code Generation
//////////////////////////////////////////////////////////////
void POSTFIX_EXPR_Node::Code_Gen(FILE *fout)
{
    //for ( int i=0; i < children.size(); ++i )
     //   children[i]->Code_Gen(fout);

    // If array call
    if ( production == "ARRAY" )
    {
        // Generate code for right side first
        // Find and set the right operand of this expression
        // This expression returns the array index
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        string index = regAlloc->getRightOp();

        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        string arrayBase = regAlloc->getLeftOp();
        regAlloc->setArrayFlag(true);
        string temp = regAlloc->Get_Register("temp1", "temp");
        fprintf(fout, "\tmul\t%s, %s, 4\n", temp.c_str(), index.c_str() );
        fprintf(fout, "\tadd\t%s, %s, %s\n", temp.c_str(), arrayBase.c_str(), temp.c_str());
        regAlloc->setArrayFlag(false);

        /*
# $t0 = address of start of A.
# $t1 = n.
mul     $t2, $t1, 4     # compute offset from the start of the array
add     $t2, $t0, $t2   # add the offset to the address of A [0].
# now $t2 = &A [n].
         */

    }
    else if ( production == "FUNC" )
    {
        func = true;
        children[0]->Code_Gen(fout);
        fprintf(fout, "\tjal\t%s\n", funcName.c_str());
        //regAlloc->findRightOp();
        string reg = regAlloc->Get_Register("RETURNREG", "func");
        func = false;
    }
    else if ( production == "FUNC_ARGS" )
    {
        /*
        for ( int i=0; i < children.size(); ++i )
            children[i]->Code_Gen(fout);
         */

        // Argument list
        regAlloc->findLeftOp();
        children[1]->Code_Gen(fout);
        string left = regAlloc->getLeftOp();
        fprintf(fout, "\tmove\t$4, %s\n", left.c_str());

        // func name
        func = true;
        children[0]->Code_Gen(fout);
        fprintf(fout, "\tjal\t%s\n", funcName.c_str());
        //regAlloc->findRightOp();

        string reg = regAlloc->Get_Register("RETURNREG", "func");
        func = false;
    }
    else
    {
        for ( int i=0; i < children.size(); ++i )
            children[i]->Code_Gen(fout);
    }
}

//////////////////////////////////////////////////////////////
// Unary expression Code Generation
//////////////////////////////////////////////////////////////
void UNARY_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Cast expression Code Generation
//////////////////////////////////////////////////////////////
void CAST_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Multiplicative expression Code Generation
//////////////////////////////////////////////////////////////
void MULTIPLICATIVE_EXPR_Node::Code_Gen(FILE *fout)
{
    /*
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
     */

    // Cast expression
    if ( children.size() == 1 )
    {
        children[0]->Code_Gen(fout);
    }
    else
    {   // Actual multiplicative expression

        string left = "";
        string right = "";

        // Note: need to take into consideration what = operator it is,
        // i.e. +=, -=, =, etc.

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        right = regAlloc->getRightOp();

        // Generate code for left side first
        // Find and set the left operand of this expression
        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        left = regAlloc->getLeftOp();

        string temp = regAlloc->Get_Register("temp", "temp");

        fprintf(fout, "\t# Multiplicative expression left *|/|% right\n");
        if ( production == "*" )
            fprintf(fout, "\tmul\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( production == "/" )
            fprintf(fout, "\tdiv\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( production == "%" )
            fprintf(fout, "\trem\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
    }
    
}

//////////////////////////////////////////////////////////////
// Additive expression Code Generation
//////////////////////////////////////////////////////////////
void ADDITIVE_EXPR_Node::Code_Gen(FILE *fout)
{
    /*
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
     */

    // Multiplicative expression
    if ( children.size() == 1 )
    {
        children[0]->Code_Gen(fout);
    }
    else
    {   // Actual additive expression

        string left = "";
        string right = "";

        // Note: need to take into consideration what = operator it is,
        // i.e. +=, -=, =, etc.

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        right = regAlloc->getRightOp();

        // Generate code for left side first
        // Find and set the left operand of this expression
        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        left = regAlloc->getLeftOp();

        string temp = regAlloc->Get_Register("temp", "temp");

        fprintf(fout, "\t# Additive expression left +/- right\n");
        if ( production == "+" )
            fprintf(fout, "\taddu\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else
            fprintf(fout, "\tsubu\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        regAlloc->Free_Constants();
    }
    
}

//////////////////////////////////////////////////////////////
// Shift expression Code Generation
//////////////////////////////////////////////////////////////
void SHIFT_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Relational expression Code Generation
//////////////////////////////////////////////////////////////
void RELATIONAL_EXPR_Node::Code_Gen(FILE *fout)
{
    if ( children.size() == 1 )
    {
	//fprintf(fout, "## one child relational\n");
        children[0]->Code_Gen(fout);
    }
    else
    {   // Actual additive expression

        string left = "";
        string right = "";
	
	//fprintf(fout, "##relational\n");
        // Note: need to take into consideration what = operator it is,
        // i.e. +=, -=, =, etc.

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        right = regAlloc->getRightOp();

        // Generate code for left side first
        // Find and set the left operand of this expression
        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        left = regAlloc->getLeftOp();

        string temp = regAlloc->Get_Register("temp", "temp");

        if ( getTypeSpecifier() == "LESS" )
            fprintf(fout, "\tslt\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( getTypeSpecifier() == "GREATER" )
            fprintf(fout, "\tsgt\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( getTypeSpecifier() == "GEQ" )
            fprintf(fout, "\tsgeu\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( getTypeSpecifier() == "LEQ" )
            fprintf(fout, "\tsleu\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
    }
} 


//////////////////////////////////////////////////////////////
// Equality expression Code Generation
//////////////////////////////////////////////////////////////
void EQUALITY_EXPR_Node::Code_Gen(FILE *fout)
{
    if ( children.size() == 1 )
    {
	//fprintf(fout, "#equality expr - one child");
        children[0]->Code_Gen(fout);
    }
    else
    {   // Actual additive expression

        string left = "";
        string right = "";

	//fprintf(fout, "#equality expr - two child");

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        right = regAlloc->getRightOp();

        // Generate code for left side first
        // Find and set the left operand of this expression
        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        left = regAlloc->getLeftOp();

        string temp = regAlloc->Get_Register("temp", "temp");

        if ( getTypeSpecifier() == "EQ_OP" )
            fprintf(fout, "\tseq\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
        else if ( getTypeSpecifier() == "NE_OP" )
            fprintf(fout, "\tsne\t%s, %s, %s\n", temp.c_str(), left.c_str(), right.c_str());
    }
 
}

//////////////////////////////////////////////////////////////
// And expression Code Generation
//////////////////////////////////////////////////////////////
void AND_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Exclusive or expression Code Generation
//////////////////////////////////////////////////////////////
void EXCLUSIVE_OR_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Inclusive or Code Generation
//////////////////////////////////////////////////////////////
void INCLUSIVE_OR_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Logical and expression Code Generation
//////////////////////////////////////////////////////////////
void LOGICAL_AND_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}


//////////////////////////////////////////////////////////////
// Logical and expression Code Generation
//////////////////////////////////////////////////////////////
void LOGICAL_OR_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}


//////////////////////////////////////////////////////////////
// Conditional expression node Code Generation
//////////////////////////////////////////////////////////////
void CONDITIONAL_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Argument expression list Code Generation
//////////////////////////////////////////////////////////////
void ARGUMENT_EXPR_LIST_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Primary expression Code Generation
//////////////////////////////////////////////////////////////
void PRIMARY_EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Assignment expression Code Generation
//////////////////////////////////////////////////////////////
void ASSIGNMENT_EXPR_Node::Code_Gen(FILE *fout)
{
    /*
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
        */

    // If conditional expression
    if ( type == "Conditional_expression" )
        children[0]->Code_Gen(fout);
    else
    {   // Actual assignment operation

        string left = "";
        string right = "";

        // Note: need to take into consideration what = operator it is,
        // i.e. +=, -=, =, etc.

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[1]->Code_Gen(fout);
        right = regAlloc->getRightOp();

        // Generate code for left side first
        // Find and set the left operand of this expression
        regAlloc->findLeftOp();
        children[0]->Code_Gen(fout);
        left = regAlloc->getLeftOp();
        fprintf(fout, "\t# Assignment expression left = right\n");
        fprintf(fout, "\tmove\t%s, %s\n", left.c_str(), right.c_str());

//        printf("%s %s\n", left.c_str(), right.c_str());
        regAlloc->Free_Register(left);
        regAlloc->Free_Temps();
        regAlloc->Free_Constants();
    }

}

//////////////////////////////////////////////////////////////
// Expression node Code Generation
//////////////////////////////////////////////////////////////
void EXPR_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Iteration statement Code Generation
//////////////////////////////////////////////////////////////
void ITERATION_STMT_Node::Code_Gen(FILE *fout)
{
	char temp_buffer[10];
	sprintf(temp_buffer, "label%d", labelNum++);
	setStartLabel(temp_buffer);
	sprintf(temp_buffer, "label%d", labelNum++);
	setEndLabel(temp_buffer);
	
	if (getLoopType() == "While")
	{
		prologue(fout); 
		//start:
		//generate assembly to check if it should jump to the end label
		//	beq $2 0 end
		//code generated from the statement
		//generate a jump to go back to the beginning
		// 	b start
		epilogue(fout);// end:
	}
	else if (getLoopType() == "DoWhile")
	{
		prologue(fout);//start:
		//code generated from the statement
		//generate assembly to check if it should jump to the end label
		//	beq $2 0 end
		//generate a jump to go back to the beginning
		// 	b start
		epilogue(fout);//end:
	}
	else if (getLoopType() == "For")  //there are 8 kinds of for loop
	{
		//initialize
		if (EXPR_A != NULL) EXPR_A->Code_Gen(fout);
		//for_loop: 	
		prologue(fout);
		
		if (EXPR_B != NULL) EXPR_B->Code_Gen(fout);
		fprintf(fout, "\n\tbeqz\t return_reg %s",getEndLabel().c_str());
		
		if (STMT!= NULL) STMT->Code_Gen(fout);
	
		//expr_c add/subtract	
		if (EXPR_C != NULL) EXPR_C->Code_Gen(fout);
		//	j	for_loop
		//end_for:	
		epilogue(fout);

	}

}

//////////////////////////////////////////////////////////////
// Statement Code Generation
//////////////////////////////////////////////////////////////
void STATEMENT_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Selection statement Code Generation
//////////////////////////////////////////////////////////////
void SELECTION_STMT_Node::Code_Gen(FILE *fout)
{
	char temp_buffer[10];
	sprintf(temp_buffer, "label%d", labelNum++);
	setStartLabel(temp_buffer);
        regAlloc->Free_Constants();
        regAlloc->Free_Temps();
	sprintf(temp_buffer, "label%d", labelNum++);
	setEndLabel(temp_buffer);
	
	//add code generated from the first child here (expr)
	//..........
	//..........
	ASTNode *temp;
	temp = getChild(0);
	temp->Code_Gen(fout);

	//start_label : else
	//end_label   : end
	if (getTypeSpecifier() == "NO_ELSE")
	{
		prologue(fout);
		//add code generated from the second child here (stmt)
		//.........
		//.........
		temp = getChild(1);
		temp->Code_Gen(fout);
		epilogue(fout);
	}
	else if (getTypeSpecifier() == "ELSE")
	{
		//prologue will call expr to gen code
		prologue(fout);
		//add code generated from the if part here (stmt1)
		//.........
		//.........
		temp = getChild(1);
		temp->Code_Gen(fout);

                regAlloc->Free_Constants();
                regAlloc->Free_Temps();
		//write code for the middle part of the else
		//like: 		b	if_end_label
		fprintf(fout, "\n\tb\t%s", getEndLabel().c_str());
		fprintf(fout, "\n%s:", getStartLabel().c_str());
		//	if_else_label:
		// 	fprintf(fout, "\t b %s\n", getEndLabel().c_str());
		//	fprintf(fout, "\t %s\n", getStartLabel().c_str());
		//add code generated from the else part here (stmt2)
		//.........
		//.........
		temp = getChild(2);
		temp->Code_Gen(fout);

		epilogue(fout);
	}
}

//////////////////////////////////////////////////////////////
// Labeled statement Code Generation
//////////////////////////////////////////////////////////////
void LABELED_STMT_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Compound statement Code Generation
//////////////////////////////////////////////////////////////
void COMPOUND_STMT_Node::Code_Gen(FILE *fout)
{
    for ( int i=0; i < children.size(); ++i )
        children[i]->Code_Gen(fout);
}

//////////////////////////////////////////////////////////////
// Return node Code Generation
//////////////////////////////////////////////////////////////
void RETURN_Node::Code_Gen(FILE *fout)
{
    // No return expression
    if ( children.size() == 0 )
        return;
    else
    {
        // Return statement consists of a single child node
        // with the return expression

        string right="";

        // Generate code for right side first
        // Find and set the right operand of this expression
        regAlloc->findRightOp();
        children[0]->Code_Gen(fout);
        right = regAlloc->getRightOp();
        fprintf(fout, "\tmove\t$2,%s\n", right.c_str());
    }
}
