#ifndef DERIVEDNODE_H
#define DERIVEDNODE_H

#include "highlevelnodes.h"
#include "stack.h"

class COMPOUND_STMT_Node: public ASTNode
{
    public:
        COMPOUND_STMT_Node();
        ~COMPOUND_STMT_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        bool addDecl(ASTNode *leaf);
        ASTNode* getDecl();

    private:
        //vector <ASTNode*> decls;
        ASTNode* decls;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
                ar & decls;
            }
        // End code for serialization
};

class LABELED_STMT_Node: public ASTNode
{
    public:
        LABELED_STMT_Node();
        ~LABELED_STMT_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        void setProduction(string);
    private:
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
                ar & production;
            }
        // End code for serialization

};

class SELECTION_STMT_Node: public ASTNode
{
    public:
        SELECTION_STMT_Node();
        ~SELECTION_STMT_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
	void prologue(FILE *);
	void epilogue(FILE *);

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class STATEMENT_Node: public ASTNode
{
    public:
        STATEMENT_Node();
        ~STATEMENT_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
	void prologue(FILE *fout);
	void epilogue(FILE *fout);

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class ITERATION_STMT_Node: public ASTNode
{
    public:
        ITERATION_STMT_Node();
        ~ITERATION_STMT_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        void addBranch(ASTNode *_branch, string _branchType);
        void setLoopType(string _loop);
        string getLoopType();
	void prologue(FILE *fout);
	void epilogue(FILE *fout);

    private:
        // The type of these pointers needs to be changed
        // later on
        ASTNode *EXPR_A;
        ASTNode *EXPR_B;
        ASTNode *EXPR_C;
        ASTNode *STMT;

        string loop;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & EXPR_A;
            ar & EXPR_B;
            ar & EXPR_C;
            ar & STMT;
            ar & loop;
        }
        // End code for serialization
};

class EXPR_Node: public ASTNode
{
    public:
        EXPR_Node();
        ~EXPR_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class ASSIGNMENT_EXPR_Node: public ASTNode
{
    public:
        ASSIGNMENT_EXPR_Node();
        ~ASSIGNMENT_EXPR_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        void setProduction(string);

    private:
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & production;
        }
        // End code for serialization
};


class PRIMARY_EXPR_Node: public ASTNode
{
    public:
        PRIMARY_EXPR_Node();
        ~PRIMARY_EXPR_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class ARGUMENT_EXPR_LIST_Node: public ASTNode
{
    public:
        ARGUMENT_EXPR_LIST_Node();
        ~ARGUMENT_EXPR_LIST_Node ();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class CONDITIONAL_EXPR_Node: public ASTNode{    
    public:        
        CONDITIONAL_EXPR_Node();        
        ~CONDITIONAL_EXPR_Node();       
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

/*
class ASSIGNMENT_OPERATOR_Node: public ASTNode{
    public:
        ASSIGNMENT_OPERATOR_Node(); 
        ~ASSIGNMENT_OPERATOR_Node(); 
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
    private:		
        // 1: operator equal
        // 2: MUL		
        // // 3: DIV		
        // // 4: MOD
        // // 5: ADD
        // // 6: SUB
        // // 7: LEFT assign
        // // 8: RIGHT assign		
        int pro;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
                ar & pro;
            }
        // End code for serialization
};
*/

class LOGICAL_OR_EXPR_Node: public ASTNode{    
    public:  
        LOGICAL_OR_EXPR_Node(); 
        ~LOGICAL_OR_EXPR_Node();  
        void Code_Gen(FILE *fout); 
        void DumpOut(FILE *fout);
    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class INCLUSIVE_OR_EXPR_Node: public ASTNode{   
    public:  
        INCLUSIVE_OR_EXPR_Node();  
        ~INCLUSIVE_OR_EXPR_Node();  
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 
    private:	

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class EXCLUSIVE_OR_EXPR_Node: public ASTNode{   
    public:  
        EXCLUSIVE_OR_EXPR_Node();  
        ~EXCLUSIVE_OR_EXPR_Node();  
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 

    private:	

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class AND_EXPR_Node: public ASTNode{   
    public:  
        AND_EXPR_Node();  
        ~AND_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 

    private:	

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};


class LOGICAL_AND_EXPR_Node: public ASTNode{   
    public:  
        LOGICAL_AND_EXPR_Node();  
        ~LOGICAL_AND_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 

    private:	

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class EQUALITY_EXPR_Node: public ASTNode{   
    public:  
        EQUALITY_EXPR_Node();  
        ~EQUALITY_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 
        void setProduction(char *);

    private:
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};

class RELATIONAL_EXPR_Node: public ASTNode{   
    public:  
        RELATIONAL_EXPR_Node();  
        ~RELATIONAL_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 
        void setProduction(char *);

    private:
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};

class SHIFT_EXPR_Node: public ASTNode{   
    public:  
        SHIFT_EXPR_Node();  
        ~SHIFT_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 
        void setProduction(char *);
    private:
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};

class ADDITIVE_EXPR_Node: public ASTNode{   
    public: 
        ADDITIVE_EXPR_Node();
        ~ADDITIVE_EXPR_Node();
        void Code_Gen(FILE *fout); 
        void DumpOut(FILE *fout);
        void setProduction(char *);
    private:	
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};

class MULTIPLICATIVE_EXPR_Node: public ASTNode{
    public:  
        MULTIPLICATIVE_EXPR_Node();
        ~MULTIPLICATIVE_EXPR_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);  
        void setProduction(char *);

    private:		
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};

class CAST_EXPR_Node: public ASTNode{
    public:
        CAST_EXPR_Node();  
        ~CAST_EXPR_Node(); 
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout); 
        void setCastType(string _castType); 
        string getCastType(); 

    private:
        string castType;
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
                ar & castType;
            }
        // End code for serialization
};

class UNARY_EXPR_Node: public ASTNode{ 
    public:  
        UNARY_EXPR_Node();
        ~UNARY_EXPR_Node(); 
        void Code_Gen(FILE *fout);  
        void DumpOut(FILE *fout); 
        void setProduction(char *);
        void setOperator(char *);

    private:		
        int pro;
        string production;
        string opt;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
            ar & opt;
        }
        // End code for serialization
};

/*
class UNARY_OPERATOR_Node: public ASTNode{	
    public:		
        UNARY_OPERATOR_Node();	
        ~UNARY_OPERATOR_Node();		
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);	
        void setProduction(char *);

    private:	
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};
*/

class POSTFIX_EXPR_Node: public ASTNode{ 
    public:
        POSTFIX_EXPR_Node(); 
        ~POSTFIX_EXPR_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout); 
        void setProduction(char *);

    private:	
        int pro;
        string production;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & pro;
            ar & production;
        }
        // End code for serialization
};


class STRING_Node: public ASTNode
{
    public:
        STRING_Node ();
        ~STRING_Node ();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization
};

class IDENTIFIER_Node: public ASTNode
{
    public:
        IDENTIFIER_Node ();
        ~IDENTIFIER_Node ();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        void setSize(int _size);
        int getSize();
        void setScope(string _scope="global");
        string getScope();

    private:
        int line;
        int size;
        string scope;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & line;
            ar & size;
            ar & scope;
        }
        // End code for serialization
};

class CONSTANT_Node: public ASTNode
{
    public:
        CONSTANT_Node ();
        ~CONSTANT_Node ();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization

};

class RETURN_Node: public ASTNode
{
    public:
        RETURN_Node ();
        ~RETURN_Node ();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:
        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
            void serialize(Archive & ar, unsigned int version)
            {
                // serialize base class information
                ar & boost::serialization::base_object<ASTNode>(*this);
            }
        // End code for serialization

};

#endif
