
#include "highlevelnodes.h"

extern int tabCount;


///////////////////////////////////////////////////////////
// FUNC_Node Implementation
///////////////////////////////////////////////////////////
FUNC_Node::FUNC_Node() :  ASTNode(),stackSize(32)
{
    _return=0;
    returnType="";
}

FUNC_Node::~FUNC_Node()
{
    for ( int i=0; i < decls.size(); ++i )
        delete decls[i];
    decls.clear();
    params.clear();
}

void FUNC_Node::addParams(vector <ASTNode*>*_params)
{
    FUNCParam tempParam;
    // add list of parameters
    for ( int i=0; i < _params->size(); ++i )
    {
        tempParam.name = _params->at(i)->getName();
        tempParam.type = _params->at(i)->getTypeSpecifier();
        tempParam.value = _params->at(i)->getValue();
        tempParam.size = _params->at(i)->paramSize;
        params.push_back(tempParam);
    }
}

void FUNC_Node::addParam(string name, string type, string value)
{
    FUNCParam fParam;
    fParam.name = name;
    fParam.type = type;
    fParam.value = value;
}

void FUNC_Node::setReturnType(char *_returnType)
{
    returnType=_returnType;
}

void FUNC_Node::setReturnType(string _returnType)
{
    returnType=_returnType;
}

string FUNC_Node::getReturnType()
{
    return returnType;
}

void FUNC_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();

    // Output my tag
    fprintf(fout, "%s<%s name=\"%s\">\n", tab.c_str(), type.c_str(), name.c_str());

    ++tabCount;
    string childTab = tab + "\t";
    // Output my data
    //
    // Function return Type
    if ( returnType.size() > 0 )
        fprintf(fout, "%s<return type=\"%s\" />\n", childTab.c_str(), 
                returnType.c_str(), type.c_str());

    // Output parameters
    for ( int i=0; i < params.size(); ++i )
    {
        fprintf(fout, "%s<param name=\"%s\" type=\"%s\" size=\"%d\" />\n", childTab.c_str(), 
                params[i].name.c_str(), params[i].type.c_str(), params[i].size);
    }

    // Output decls
    for ( int i=0; i < decls.size(); ++i )
    {
        decls[i]->DumpOut(fout);
    }

    // Output children
    for ( int i=0; i < children.size(); ++i )
    {
        children[i]->DumpOut(fout);
    }
    --tabCount;

    fprintf(fout, "%s</%s>\n", tab.c_str(), type.c_str());
}

bool FUNC_Node::addDecl(ASTNode *leaf)
{
    if ( leaf != NULL )
    {
        decls.push_back(leaf);
        return true;
    }
    else
    {
        printf("Warning: Adding NULL Declaration leaf to AST!!\n");
        return false;
    }
}


///////////////////////////////////////////////////////////
// DECL_Node Implementation
///////////////////////////////////////////////////////////
DECL_Node::DECL_Node() : ASTNode()
{
}

DECL_Node::~DECL_Node()
{
}

void DECL_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Declaration>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Declaration>\n", tab.c_str());
}


///////////////////////////////////////////////////////////
// Statement Node Implementation
///////////////////////////////////////////////////////////
STMTNode::STMTNode() : ASTNode()
{
}

STMTNode::~STMTNode()
{
}

void STMTNode::DumpOut(FILE* fout)
{
}

void STMTNode::Code_Gen(FILE *fout)
{
}

void STMTNode::addStmt(STMTNode *_stmt)
{
    stmts.push_back(_stmt);
}

///////////////////////////////////////////////////////////
// PROGNODE Implementation
///////////////////////////////////////////////////////////
PROGNode::PROGNode() : ASTNode()
{
}

PROGNode::~PROGNode()
{
    int i=0;
    for ( i=0; i < funcs.size(); ++i )
        delete funcs[i];

    for ( i=0; i < decls.size(); ++i )
        delete decls[i];

    funcs.clear();
    decls.clear();
}

bool PROGNode::addDecl(ASTNode *leaf)
{
    if ( leaf != NULL )
    {
        decls.push_back(leaf);
        return true;
    }
    else
    {
        printf("Warning: Adding NULL Declaration leaf to AST!!\n");
        return false;
    }
}

bool PROGNode::addFunc(ASTNode *leaf)
{
    if ( leaf != NULL )
    {
        funcs.push_back(leaf);
        return true;
    }
    else
    {
        printf("Warning: Adding NULL Function leaf to AST!!\n");
        return false;
    }
}

void PROGNode::DumpOut(FILE *fout)
{
    tabCount = 1;
    fprintf(fout, "<%s>\n", type.c_str());

    // Output decls
    for ( int i=0; i < decls.size(); ++i )
        decls[i]->DumpOut(fout);

    // Output funcs
    for ( int i=0; i < funcs.size(); ++i )
        funcs[i]->DumpOut(fout);

    fprintf(fout, "</%s>\n", type.c_str());

}
