#ifndef HIGHLEVELNODE_H
#define HIGHLEVELNODE_H

#include "astnode.h"

class IDENTIFIER_Node;
class FUNC_Node: public ASTNode
{
    public:
        FUNC_Node();
        ~FUNC_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

        void setReturnType(string _retType);
        void setReturnType(char *_retType);
        string getReturnType();

        // Adds a parameter
        void addParam(string name, string type, string value="");
        // vector of parameters
        void addParams(vector <ASTNode*>*_params);

        // Decls
        bool addDecl(ASTNode *leaf);

        // Compute function prologue
        void prologue(FILE *fout);
        void epilogue(FILE *fout);

    private:
        vector<FUNCParam>params;
        vector <ASTNode *>decls;
        int _return;
        string returnType;
        int stackSize;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & params;
            ar & decls;
            ar & _return;
            ar & returnType;
        }
        // End code for serialization
};


class DECL_Node: public ASTNode
{
    public:
        DECL_Node();
        ~DECL_Node();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);

    private:
        vector<Declaration> decls;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & decls;
        }
        // End code for serialization
};

class STMTNode: public ASTNode {
    public:
        STMTNode();
        ~STMTNode();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *fout);
        void addStmt(STMTNode *_stmt);

    private:
        // statements
        vector<STMTNode *> stmts;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & stmts;
        }
        // End code for serialization
};

class PROGNode: public ASTNode
{
    public:
        PROGNode();
        ~PROGNode();
        void Code_Gen(FILE *fout);
        void DumpOut(FILE *);

        // Adds a child to the node
        bool addDecl(ASTNode *leaf);
        bool addFunc(ASTNode *leaf);

    private:
        vector <ASTNode *>funcs;
        vector <ASTNode *>decls;

        // Code for serialization
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, unsigned int version)
        {
            // serialize base class information
            ar & boost::serialization::base_object<ASTNode>(*this);
            ar & funcs;
            ar & decls;
        }
        // End code for serialization
};

#endif
