#ifndef RBTREE_CPP
#define RBTREE_CPP

#define BLACK 0
#define RED 1

#include <string.h>
#include "rbtree.h"

extern int stDebugLevel;
enum {DBGLV0, DBGLV1, DBGLV2};

node::node()
{
    lineNumber = 0;
    name = "default";
    left = NULL;
    right = NULL;
    parent = NULL;
    color = RED;
    checked = 0;
    type = 0;
    value = 0;
    size = -1;

    function.returnType=0;
    function.cursor=0;
    function.numParams=0;

    scope=0;
    offset=0;
}

node::node(int v, string n, int _type, int s)
{
    lineNumber = v;
    name = n;
    color = RED;
    checked = 0;
    type = _type;
    left = NULL;
    right = NULL;
    parent = NULL;
    value = 0;
    size = s;

    function.returnType=0;
    function.cursor=0;
    function.numParams=0;

    scope=0;
    offset=0;
}

node::~node()
{}

void node::setType(int _type)
{
    type = _type;
}

void node::setValue(int _value)
{
    value = _value;
}

void node::insertParam(string _name, int _type, int _size)
{
    Parameter tempParam;
    tempParam.name = _name;
    tempParam.type = _type;
    tempParam.size = _size;
    function.parameters.push_back(tempParam);
}

void node::setReturnType(int _type)
{
    function.returnType=_type;
}

int node::getParameters()
{
    for ( int i=0; i < function.parameters.size(); ++i )
    {
        printf("param: %s %s\n", function.parameters[i].name.c_str(), 
                typeMap[function.parameters[i].type].c_str());
    }
    return 0;
}

// Get variable name
string node::getName()
{
    return name;
}

int node::getValue()
{
    return value;
}

// Increases the number of times a variable is used
// after decl
void node::check()
{
    ++checked;
}

// Get variable type
int node::getType()
{
    return type;
}

// Get variable size
int node::getSize()
{
    return size;
}

// Set array size
void node::setSize(int _size)
{
    size=_size;
}


// Get line number of variable decl
int node::getLine()
{
    return lineNumber;
}

// Check to see if variable was decl and used
int node::getCheck()
{
    return checked;
}

// Get number of parameters in function
int node::getFuncArgc()
{
    return function.parameters.size();
}

// Get the i(th) function parameter
Parameter* node::getParameter(int i)
{
    if ( i < function.parameters.size() )
        return &(function.parameters[i]);
    else
    {
        fprintf(stderr, "Parameter index out of range!\n");
        return NULL;
    }
}

// Get the i(th) function parameter type
int node::getParameterType(int i)
{
    if ( i < function.parameters.size() )
        return function.parameters[i].type;
    else
    {
        fprintf(stderr, "Parameter index out of range!\n");
        return 0;
    }
}

void node::resetParamCursor()
{
   function.cursor=0;
}

void node::showNode()
{
    cout<<"lineNumber="<<lineNumber<<" name="<<name<<endl;;
}


node& node::operator=(const node &rSide)
{
    if ( &rSide == this )
        return *this;

    parent = rSide.parent;
    left = rSide.left;
    right = rSide.right;
    lineNumber = rSide.lineNumber;
    value = rSide.value;
    type = rSide.type;
    size = rSide.size;

    function.returnType = rSide.function.returnType;
    function.cursor = rSide.function.cursor;
    function.numParams = rSide.function.numParams;

    for ( int i=0; i < rSide.function.parameters.size(); ++i )
        function.parameters.push_back(rSide.function.parameters[i]);


    name = rSide.name;
    color = rSide.color;
    checked = rSide.checked;

    offset = rSide.offset;
    scope = rSide.scope;

    return *this;
}

//////////////////////////////////////////////
// Methods needed for the code generation ST
//////////////////////////////////////////////
int node::getScope()
// Get variable scope level
{
    return scope;
}
// Set scope level
void node::setScope(int _scope)
{
    scope=_scope;
}

// Get variable stack offset
int node::getOffset()
{
    return offset;
}
// Set variable stack offset
void node::setOffset(int _offset)
{
    offset=_offset;
}

// Set variable stack offset and scope
void node::setStackInfo(int _offset, int _scope)
{
    offset=_offset;
    scope=_scope;
}

//////////////////////////////////////////////



/***************************************************
*  RB TREE
****************************************************/
rbtree::rbtree()
{
    root = NULL;
    size=0;
}

rbtree::~rbtree()
{
    clear();
}

node* rbtree::grandparent(node * n)
{
    //   if (n == NULL) return ;
    //   if (n->parent == NULL) return;
    return n->parent->parent;
}

node* rbtree::uncle(node * n)
{
    if (n->parent == grandparent(n)->left)
        return grandparent(n)->right;
    else
        return grandparent(n)->left;
}

node* rbtree::insert(node *z)
{
    node *retNode = z;
    // Place node in corresponding branch
    placeNode(z);

    return retNode;
}
node* rbtree::insert(int v, string n, int _type)
{
    node * z = new node(v, n, _type);
    node *retNode = z;
    // Place node in corresponding branch
    placeNode(z);

    return retNode;
}

void rbtree::placeNode(node *z)
{
    node * y = NULL; 
    node * x = root;

    ++size;
    while (x != NULL)
    {
        y = x;
        if (z->name < x->name)
            x = x->left;
        else
            x = x->right;
    }

    z->parent = y;

    if (y == NULL)
    {
        root = z;
    }		
    else if (z->name < y->name)
        y->left = z;
    else 	y->right = z;

    if (root == z) 
    {
        root->color = BLACK;
        return;
    }

    if (z->parent->parent == NULL) 
    {
        //cout<<"first level"<<endl;
        return;
    }

    //RB-FIXUP
    while (z->parent!=NULL && z->parent->color == RED)
    {
        //printf("fixing \n");
        if (z->parent == z->parent->parent->left)
        {
            //printf("left grandchild \n");
            y = z->parent->parent->right;
            if (y!=NULL && y->color == RED)
            {
                //printf("uncle is red\n");
                z->parent->color = BLACK;
                y->color = BLACK;
                //printf("color grandparent to red\n");
                z->parent->parent->color = RED;
                //printf("set z to be grandparent\n");
                z = z->parent->parent;
            }	

            else
            {
                if (z->parent!=NULL && z == z->parent->right)
                {
                    z = z->parent;
                    left_rotate( z );
                }
                z->parent->color = BLACK;
                z->parent->parent->color = RED;
                right_rotate( z->parent->parent );
            }
        }
        else 
        {
            //printf("right grandchild\n");
            y = z->parent->parent->left;
            if (y!=NULL && y->color == RED)
            {
                //showTree();
                z->parent->color = BLACK;
                y->color = BLACK;
                z->parent->parent->color = RED;
                z = z->parent->parent;
                //printf("changes .... \n");
                //showTree();
            }
            else
            {
                //showTree();
                if (z->parent!=NULL && z == z->parent->left)
                {
                    z = z->parent;
                    right_rotate( z );
                }
                z->parent->color = BLACK;
                z->parent->parent->color = RED;
                left_rotate( z->parent->parent );
            }
        }
        root->color = BLACK;
    }
}

/*
node* rbtree::insert(int v, string n, int _type)
{
    node * y = NULL; 
    node * x = root;
    node * z = new node(v, n, _type);
    node *retNode = z;

    ++size;
    while (x != NULL)
    {
        y = x;
        if (z->name < x->name)
            x = x->left;
        else
            x = x->right;
    }

    z->parent = y;

    if (y == NULL)
    {
        root = z;
    }		
    else if (z->name < y->name)
        y->left = z;
    else 	y->right = z;

    if (root == z) 
    {
        root->color = BLACK;
        return retNode;
    }

    if (z->parent->parent == NULL) 
    {
        //cout<<"first level"<<endl;
        return retNode;
    }


    //RB-FIXUP
    while (z->parent!=NULL && z->parent->color == RED)
    {
        //printf("fixing \n");
        if (z->parent == z->parent->parent->left)
        {
            //printf("left grandchild \n");
            y = z->parent->parent->right;
            if (y!=NULL && y->color == RED)
            {
                //printf("uncle is red\n");
                z->parent->color = BLACK;
                y->color = BLACK;
                //printf("color grandparent to red\n");
                z->parent->parent->color = RED;
                //printf("set z to be grandparent\n");
                z = z->parent->parent;
            }	

            else
            {
                if (z->parent!=NULL && z == z->parent->right)
                {
                    z = z->parent;
                    left_rotate( z );
                }
                z->parent->color = BLACK;
                z->parent->parent->color = RED;
                right_rotate( z->parent->parent );
            }
        }
        else 
        {
            //printf("right grandchild\n");
            y = z->parent->parent->left;
            if (y!=NULL && y->color == RED)
            {
                //showTree();
                z->parent->color = BLACK;
                y->color = BLACK;
                z->parent->parent->color = RED;
                z = z->parent->parent;
                //printf("changes .... \n");
                //showTree();
            }
            else
            {
                //showTree();
                if (z->parent!=NULL && z == z->parent->left)
                {
                    z = z->parent;
                    right_rotate( z );
                }
                z->parent->color = BLACK;
                z->parent->parent->color = RED;
                left_rotate( z->parent->parent );
            }
        }
        root->color = BLACK;
    }

    return retNode;
}
*/

void rbtree::left_rotate(node * x) 
{
    //printf("left rotate \n");
    node * y;
    y = x->right;
    /* Turn y's left sub-tree into x's right sub-tree */
    x->right = y->left;
    if ( y->left != NULL )
        y->left->parent = x;
    /* y's new parent was x's parent */
    y->parent = x->parent;
    /* Set the parent to point to y instead of x */
    /* First see whether we're at the root */
    if ( x->parent == NULL ) root = y;
    else
        if ( x == (x->parent)->left )
            /* x was on the left of its parent */
            x->parent->left = y;
        else
            /* x must have been on the right */
            x->parent->right = y;
    /* Finally, put x on y's left */
    y->left = x;
    x->parent = y;
}

void rbtree::right_rotate(node * x) 
{
    //printf("right rotate \n");
    node * y;
    y = x->left;
    x->left = y->right;
    if ( y->right != NULL )
        y->right->parent = x;
    y->parent = x->parent;
    if ( x->parent == NULL ) root = y;
    else
        if ( x == (x->parent)->right )
            x->parent->right = y;
        else
            x->parent->left = y;
    y->right = x;
    x->parent = y;
}


void rbtree::insertSub(int v, string n, node *& p, node *&t)
{
    if (p == NULL) {
        p = new node (v, n);
        p->parent = t;
        return ;
    }

    else {
        if (n < p->name) 
            insertSub(v, n, p->left, p);
        else
            insertSub(v, n, p->right, p);
    }
}

node * rbtree::search(string n)
{
    return searchSub(n, root);
}

node * rbtree::searchSub(string n, node *p)
{
    if (p==NULL) return NULL;

    if (n == p->name)
    {
	p->check();
        return p;
    }
    else if (n < p->name)
    {
        return searchSub(n, p->left);
    }
    else if (n > p->name)
    {
        return searchSub(n, p->right);
    }
}

void rbtree::clear()
{
    clearSub(root);
}

void rbtree::clearSub(node *&p)
{
    if (p==NULL) return;
    if (p->left != NULL) clearSub(p->left);
    if (p->right != NULL) clearSub(p->right);
    delete p;
}

void rbtree::dump(int l, FILE *f)
{
    dumpSub(root, 0, f);
}

void rbtree::dumpSub(node *p, int level, FILE *f)
{
    int j;
    if (p == NULL) return;  
    if ( p != 0 )
    {
        dumpSub(p->right,level+1,f);         // Output right subtree
        for ( j = 0 ; j < level ; j++ )    // Tab over to level
            fprintf(f, "\t\t");

        // typeMap -> typeMap[INT] prints the string "INT"

        if ( stDebugLevel == DBGLV2 )
            fprintf(f, " #%s:%s, L:%d V:%d", p->name.c_str(), typeMap[p->getType()].c_str(), p->lineNumber, p->value);
        else
            fprintf(f, " #%s:%s, L:%d", p->name.c_str(), typeMap[p->getType()].c_str(), p->lineNumber);


        if ( ( p->left != 0 ) &&           // Output "connector"
                ( p->right != 0 ) )
            fprintf(f, " <");
        else if ( p->right != 0 )
            fprintf(f, "/");
        else if ( p->left != 0 )
            fprintf(f,"\\");
        fprintf(f, "\n");
        dumpSub(p->left,level+1,f);          // Output left subtree
    }

    if ( p == NULL ) 
        return;

    return;
    dumpSub(p->left, level, f);
    //cout<<p->name<<"\t"<<l<<endl;
    //f<<p->name<<"\t"<<l<<endl;
    fprintf(f, "%s,%d\n",p->name.c_str(),level);
    dumpSub(p->right, level, f);
}

void rbtree::showTree()
{
    cout << endl;
    showSub(root, 1);
    cout << endl;
}

void rbtree::showSub(node * p, int level)
{
    int j;
    if (p == NULL) return;  
    if ( p != 0 )
    {
        showSub(p->right,level+1);         // Output right subtree
        for ( j = 0 ; j < level ; j++ )    // Tab over to level
            cout << "\t\t";
        //cout << " #" << p->lineNumber<<" "<<p->name;
        //printf(" #%d,%s", p->lineNumber, p->name);
        printf(" #%d,%s", p->lineNumber, p->name.c_str());
        if (p->color == RED) cout<<" RED";
        else cout<<" BLACK";

        if ( ( p->left != 0 ) &&           // Output "connector"
                ( p->right != 0 ) )
            cout << " <";
        else if ( p->right != 0 )
            cout << "/";
        else if ( p->left != 0 )
            cout << "\\";
        cout << endl;
        showSub(p->left,level+1);          // Output left subtree
    }
}

void rbtree::treeWalk(node* cursor, node** _nodes, int &index)
{
    if ( cursor != NULL )
    {
        treeWalk(cursor->left, _nodes, index);
        _nodes[index]=cursor;
        ++index;
        treeWalk(cursor->right, _nodes, index);
    }
}


node** rbtree::foreach()
{
    int index=0;
    node** _nodes = new node*[size];
    treeWalk(root, _nodes, index);
    return _nodes;
}

int rbtree::getSize()
{
    return size;
}

#endif
