#ifndef RBTREE_H
#define RBTREE_H

#include <iostream>
#include <string>
#include <vector>
using namespace std;

enum { 
    _VOID, 
    _CHAR, 
    _SHORT, 
    _INT, 
    _LONG, 
    _FLOAT, 
    _DOUBLE, 
    _SIGNED, 
    _UNSIGNED,
    _FUNCPROTO,
    _FUNCTION,
    _ENUM,
    _STRING,
    _STRUCT
};

static string typeMap[]={
    "VOID", 
    "CHAR", 
    "SHORT", 
    "INT", 
    "LONG", 
    "FLOAT", 
    "DOUBLE", 
    "SIGNED", 
    "UNSIGNED",
    "FUNCPROTO",
    "FUNC",
    "ENUM",
    "STRING",
    "STRUCT"
};

enum {
    _CHAR_SIZE = sizeof(char),
    _SHORT_SIZE = sizeof(short), 
    _INT_SIZE = sizeof(int), 
    _LONG_SIZE = sizeof(long), 
    _FLOAT_SIZE = sizeof(float), 
    _DOUBLE_SIZE = sizeof(double), 
    _SIGNED_SIZE = sizeof(signed), 
    _UNSIGNED_SIZE = sizeof(unsigned)
};

enum{
    GLOBAL,
    LOCAL,
    PARAM1,
    PARAM2,
    FUNC,
    RET
};


struct Parameter
{
    string name;
    int type;
    int size;
};

struct funcVars
{
    int returnType;
    vector<Parameter>parameters;
    int cursor;
    int numParams;
};

class node{
    public:
        node();
        node(int v, string n, int _type=0, int s=-1);
        ~node();
        void showNode();
        void check();
        int getLine();
        int getType();
        int getCheck();
        string getName();
        void setType(int _type);
        void setReturnType(int _type);

        // Methods for function nodes
        void insertParam(string _name, int _type, int _size=-1);
        int getParameters();
        int getFuncArgc();
        void resetParamCursor();
        Parameter* getParameter(int i);
        int getParameterType(int i);

        int getValue();
        void setValue(int _value);

        // Method for arrays
        void setSize(int _size);
        int getSize();

        // Methods needed in code generation
        int getScope();
        void setScope(int _scope);
        int getOffset();
        void setOffset(int _offset);

        void setStackInfo(int _offset, int _scope);

        node& operator=(const node &rSide);

    private:
        node * parent;
        node * left;
        node * right;
        int lineNumber;
        int value;
	int type;

        // for arrays
	int size;

        funcVars function;
		 
        string name;
        int color;
	int checked;
        friend class rbtree;

        // stack offset
        int offset;
        // scope level
        int scope;
};

class rbtree
{
    public:
        rbtree();
        ~rbtree();
        node *  grandparent(node * n);
        node * uncle(node * n);
        node* insert(node *z);
        node* insert(int, string, int);
        // Places node in tree directly
        void placeNode(node *z);
        void insertSub(int, string, node *&, node *&);
        node * search( string);
        node * searchSub( string, node *);
        void left_rotate(node *);
        void right_rotate(node *);
        void remove();
        void clear();
        void clearSub(node *&);
        void dump(int l, FILE *); 
        void dumpSub(node *p, int l, FILE *); 
        void showTree();
        void showSub(node *p, int l);

        node** foreach();
        int getSize();
        void treeWalk(node* cursor, node** _nodes, int &index);

    private:
        rbtree* previous;
        rbtree* next;
        node * root;
        int size;

        friend class Stack;
};

#endif
