#include "regalloc.h"

extern FILE *f_code;
extern Stack *codegen_st;
//static node *tempNode=NULL;

RegisterAllocator::RegisterAllocator()
{
    char temp[32];
    for ( int i=8; i <= 15; ++i )
    {
        register_table[i][VARIABLE]="";
    }
    register_table[24][VARIABLE]="";
    register_table[25][VARIABLE]="";

    for ( int i=0; i < 32; ++i )
    {
        sprintf(temp, "$%d", i);
        register_names[i]=temp;
    }

    leftOperand="";
    rightOperand="";
    findOp = false;
    lastRegister="";
    arrayIndex=0;
    arrayFlag=false;
}

RegisterAllocator::~RegisterAllocator()
{
}


/**
 * Gets a register available.
 * Writes the assembly for the allocation.
 * @varName name of variable that needs a register
 */
string RegisterAllocator::Get_Register(string varName, string type)
{
    char temp[128];

    int empty=-1;

    if ( type == "func" )
    {
        fprintf(f_code, "\t# Req Func Var\n");
        empty=2;
    }
    else{
        for ( int i=8; i <= 15; ++i )
        {
            if ( register_table[i][VARIABLE] == varName )
            {
                //printf("reusing %s %s\n", register_names[i].c_str(), varName.c_str());
                setOperand(i,arrayFlag);
                return register_names[i];
            }
            else if ( empty == -1 && register_table[i][VARIABLE] == "" )
                empty=i;
        }

        // If all registers are empty, write something to memory
        if ( empty == -1 )
        {
            //printf("Need to Spill!!\n");
            empty = 8;

            for ( int i=8; i <= 15; ++i )
            {
                if ( register_table[i][TYPE] == "constant" )
                {
                    register_table[i][VARIABLE]="";
                    register_table[i][TYPE]="";
                    register_table[i][REG]="";
                    empty = i;
                }
            }
        }

        if ( type == "constant" )
        {
            fprintf(f_code, "\t# Load constant %s\n", varName.c_str());
            fprintf(f_code, "\tli\t%s, %s\n", register_names[empty].c_str(), varName.c_str());
        }
        else if ( type == "global" )
        {
            fprintf(f_code, "\t# Load global %s\n", varName.c_str());
            fprintf(f_code, "\tlw\t%s, %s\n", register_names[empty].c_str(), varName.c_str());
        }
        else if ( type == "local" )         // Local variable
        {
            int tempLevel=0;
            node *tempNode = codegen_st->search(varName, tempLevel);
            if ( tempNode == NULL )
            {
                printf("Something wrong in ST!!!\n");
                printf("Searching for %s\n", varName.c_str());
            }
            else
            {
                if ( tempNode->getScope() == PARAM1 )
                {
                    empty=4;
                }
                else if ( tempNode->getScope() == PARAM2 )
                {
                    empty=5;
                }
                else
                {
                    fprintf(f_code, "\t# Load local variable %s\n", varName.c_str());
                    fprintf(f_code, "\tlw\t%s, %d($sp)\n", register_names[empty].c_str(),
                            tempNode->getOffset());
                }
            }
        }
        else
        {           // Temp variable
            fprintf(f_code, "\t# Load temp\n");
        }
    }

    setOperand(empty,arrayFlag);

    register_table[empty][VARIABLE] = varName;
    register_table[empty][TYPE] = type;
    register_table[empty][REG] = register_names[empty];

    return register_names[empty];
}

void RegisterAllocator::Free_Constants()
{
    for ( int i=8; i <= 15; ++i ) 
    {
        if ( register_table[i][TYPE] == "constant" )
        {
            register_table[i][VARIABLE]="";
            register_table[i][TYPE]="";
            register_table[i][REG]="";
        }
    }
}

void RegisterAllocator::Free_Temps()
{
    for ( int i=8; i <= 15; ++i ) 
    {
        if ( register_table[i][TYPE] == "temp" )
        {
            register_table[i][VARIABLE]="";
            register_table[i][TYPE]="";
            register_table[i][REG]="";
        }
    }
}

void RegisterAllocator::Free_Register(string regName)
{
    int index=-1;
    // Clear only if it's global or local 
    // Search on registers
    //printf("%s\n", regName.c_str());
    for ( int i=8; i <= 15; ++i ) 
    {
    //    printf("r:%s\n", register_names[i].c_str());
        // Corresponding Register found
        if ( regName == register_names[i] )
        {
            index = i;
            break;
        }
    }

    if ( index == -1  )
        return;

//    printf("%d %s %s\n", index, register_table[index][VARIABLE].c_str(), regName.c_str());
    int tempLevel=0;
    node *tempNode = codegen_st->search(register_table[index][VARIABLE], tempLevel);
    if ( tempNode == NULL )
    {
        printf("Something wrong in ST!!!\n");
        printf("Searching for %s\n", register_table[index][VARIABLE].c_str());
    }
    else
    {
        // Global var save to stack
        if ( tempNode->getScope() == GLOBAL )
        {
            fprintf(f_code, "\t# Save global variable %s\n", register_table[index][VARIABLE].c_str());
            fprintf(f_code, "\tsw\t%s, %s\n", register_names[index].c_str(),
                    regName.c_str());
        }
        // Local var save to stack
        else if ( tempNode->getScope() == LOCAL )
        {
            fprintf(f_code, "\t# Save local variable %s\n", register_table[index][VARIABLE].c_str());
            fprintf(f_code, "\tsw\t%s, %d($sp)\n", register_names[index].c_str(),
                    tempNode->getOffset());
        }

        register_table[index][VARIABLE]="";
        register_table[index][TYPE]="";
    }
}

void RegisterAllocator::spill()
{
}

void RegisterAllocator::setOperand(int index, bool array)
{
    if ( findOp )
    {
        if ( array )
        {
            if ( findOpHistory.top() == LEFT_OPERAND )
                leftOperand = "("+register_names[index]+")";
            else
                rightOperand = "("+register_names[index]+")";
        }
        else
        {
            if ( findOpHistory.top() == LEFT_OPERAND )
                leftOperand = register_names[index];
            else
                rightOperand = register_names[index];
        }
        findOpHistory.pop();
    }
    lastRegister=register_names[index];
}

void RegisterAllocator::findLeftOp()
{
    findOpHistory.push(LEFT_OPERAND);
    findOp = true;
}

void RegisterAllocator::findRightOp()
{
    findOpHistory.push(RIGHT_OPERAND);
    findOp = true;
}

/*
void RegisterAllocator::setLastRegister(string _reg)
{
    lastRegister = _reg;
}
*/

string RegisterAllocator::getLastRegister()
{
    return lastRegister;
}

string RegisterAllocator::getLeftOp()
{
    if ( findOpHistory.empty() )
        findOp = false;
    return leftOperand;
}

string RegisterAllocator::getRightOp()
{
    if ( findOpHistory.empty() )
        findOp = false;
    return rightOperand;
}

/*
// Get array index
int node::getArrayIndex()
{
    return arrayIndex;
}

// Set array size
void node::setArrayIndex(int _arrayIndex)
{
    arrayIndex = _arrayIndex;
}
*/

// Set array size
void RegisterAllocator::setArrayFlag(bool _array)
{
    arrayFlag=_array;
}

